@@ -20,7 +20,7 @@
 
 #define QT_FEATURE_reduce_exports 1
 
-#define QT_FEATURE_no_direct_extern_access 1
+#define QT_FEATURE_no_direct_extern_access -1
 
 #define QT_FEATURE_x86intrin -1
 
@@ -74,7 +74,7 @@
 
 #define QT_FEATURE_mips_dspr2 -1
 
-#define QT_FEATURE_neon 1
+#define QT_FEATURE_neon -1
 
 #define QT_FEATURE_arm_crc32 -1
 
