@@ -6,6 +6,8 @@
 
 #include <QtCore/qglobal.h>
 
+#include <QtCore/q20utility.h>
+
 QT_BEGIN_NAMESPACE
 
 template <typename T>
@@ -20,9 +22,8 @@ public:
 
     Q_NODISCARD_CTOR
     explicit constexpr QScopedValueRollback(T &var, T value)
-        : varRef(var), oldValue(std::move(var)) // ### C++20: std::exchange(var, std::move(value))
+        : varRef(var), oldValue(q20::exchange(var, std::move(value)))
     {
-        var = std::move(value);
     }
 
 #if __cpp_constexpr >= 201907L
