@@ -4,8 +4,6 @@
 
 #define QT_FEATURE_cxx17_filesystem 1
 
-#define QT_FEATURE_eventfd -1
-
 #define QT_FEATURE_glib 1
 
 #define QT_FEATURE_inotify -1
@@ -84,6 +82,5 @@
 
 #define QT_FEATURE_permissions 1
 
-#define QT_NO_EVENTFD 1
 #define QT_NO_INOTIFY 1
 #define QT_THREADSAFE_CLOEXEC 1
