@@ -43,8 +43,11 @@ struct QDockWidgetPrivate {
 #endif
 #if QT_CONFIG(toolbar)
 #include "qtoolbararealayout_p.h"
+#include "qtoolbar.h"
 #endif
+
 #include <QtCore/qloggingcategory.h>
+#include <QtCore/qpointer.h>
 
 QT_REQUIRE_CONFIG(mainwindow);
 
@@ -90,6 +93,9 @@ public:
     bool endSeparatorMove(const QPoint &pos);
     bool windowEvent(QEvent *e);
 
+private:
+    QList<int> findSeparator(const QPoint &pos) const;
+
 #endif // QT_CONFIG(dockwidget)
 
 };
@@ -140,7 +146,7 @@ void QMainWindowLayoutSeparatorHelper<Layout>::adjustCursor(const QPoint &pos)
                 w->unsetCursor();
         }
     } else if (movingSeparator.isEmpty()) { // Don't change cursor when moving separator
-        QList<int> pathToSeparator = layout()->dockAreaLayoutInfo()->findSeparator(pos);
+        QList<int> pathToSeparator = findSeparator(pos);
 
         if (pathToSeparator != hoverSeparator) {
             if (!hoverSeparator.isEmpty())
@@ -277,10 +283,35 @@ bool QMainWindowLayoutSeparatorHelper<Layout>::windowEvent(QEvent *event)
     return false;
 }
 
+template <typename Layout>
+QList<int> QMainWindowLayoutSeparatorHelper<Layout>::findSeparator(const QPoint &pos) const
+{
+    Layout *layout = const_cast<Layout*>(this->layout());
+#if QT_CONFIG(toolbar)
+    QToolBarAreaLayout *toolBarAreaLayout = layout->toolBarAreaLayout();
+    if (toolBarAreaLayout && !toolBarAreaLayout->isEmpty()) {
+        // We might have a toolbar that is currently expanded, covering
+        // parts of the dock area, in which case we don't want the dock
+        // area layout to treat mouse events for the expanded toolbar as
+        // hitting a separator.
+        const QWidget *widget = layout->window();
+        QWidget *childWidget = widget->childAt(pos);
+        while (childWidget && childWidget != widget) {
+            if (auto *toolBar = qobject_cast<QToolBar*>(childWidget)) {
+                if (!toolBarAreaLayout->indexOf(toolBar).isEmpty())
+                    return {};
+            }
+            childWidget = childWidget->parentWidget();
+        }
+    }
+#endif
+    return layout->dockAreaLayoutInfo()->findSeparator(pos);
+}
+
 template <typename Layout>
 bool QMainWindowLayoutSeparatorHelper<Layout>::startSeparatorMove(const QPoint &pos)
 {
-    movingSeparator = layout()->dockAreaLayoutInfo()->findSeparator(pos);
+    movingSeparator = findSeparator(pos);
 
     if (movingSeparator.isEmpty())
         return false;
@@ -491,6 +522,7 @@ public:
     void removeToolBar(QToolBar *toolbar);
     void toggleToolBarsVisible();
     void moveToolBar(QToolBar *toolbar, int pos);
+    QToolBarAreaLayout *toolBarAreaLayout() { return &layoutState.toolBarAreaLayout; }
 #endif
 
     // dock widgets
